@@ -14,17 +14,26 @@
 #
 # Authors:
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2012-2013
-# - Vincent Garonne <vgaronne@gmail.com>, 2012-2018
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
 # - Yun-Pin Sun <winter0128@gmail.com>, 2013
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2020
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2015
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2015
-# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2015
+# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2015-2018
 # - Martin Barisits <martin.barisits@cern.ch>, 2016-2019
 # - Tobias Wegner <twegner@cern.ch>, 2017
 # - Brian Bockelman <bbockelm@cse.unl.edu>, 2017-2018
-# - Ruturaj Gujar, <ruturaj.gujar23@gmail.com>, 2019
-# - Jaroslav Guenther <jaroslav.guenther@gmail.com>, 2019-2020
+# - Robert Illingworth <illingwo@fnal.gov>, 2018
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
+# - Tomas Javurek <tomas.javurek@cern.ch>, 2019
+# - Brandon White <bjwhite@fnal.gov>, 2019
+# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
+# - Eric Vaandering <ericvaandering@gmail.com>, 2019
+# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -114,19 +123,20 @@ class BaseClient(object):
     TOKEN_PREFIX = 'auth_token_'
     TOKEN_EXP_PREFIX = 'auth_token_exp_'
 
-    def __init__(self, rucio_host=None, auth_host=None, account=None, ca_cert=None, auth_type=None, creds=None, timeout=600, user_agent='rucio-clients'):
+    def __init__(self, rucio_host=None, auth_host=None, account=None, ca_cert=None, auth_type=None, creds=None, timeout=600, user_agent='rucio-clients', vo=None):
         """
         Constructor of the BaseClient.
-        :param rucio_host: the address of the rucio server, if None it is read from the config file.
-        :param rucio_port: the port of the rucio server, if None it is read from the config file.
-        :param auth_host: the address of the rucio authentication server, if None it is read from the config file.
-        :param auth_port: the port of the rucio authentication server, if None it is read from the config file.
-        :param account: the account to authenticate to rucio.
-        :param use_ssl: enable or disable ssl for commucation. Default is enabled.
-        :param ca_cert: the path to the rucio server certificate.
-        :param auth_type: the type of authentication (e.g.: 'userpass', 'kerberos' ...)
-        :param creds: a dictionary with credentials needed for authentication.
-        :param user_agent: indicates the client
+        :param rucio_host: The address of the rucio server, if None it is read from the config file.
+        :param rucio_port: The port of the rucio server, if None it is read from the config file.
+        :param auth_host: The address of the rucio authentication server, if None it is read from the config file.
+        :param auth_port: The port of the rucio authentication server, if None it is read from the config file.
+        :param account: The account to authenticate to rucio.
+        :param use_ssl: Enable or disable ssl for commucation. Default is enabled.
+        :param ca_cert: The path to the rucio server certificate.
+        :param auth_type: The type of authentication (e.g.: 'userpass', 'kerberos' ...)
+        :param creds: Dictionary with credentials needed for authentication.
+        :param user_agent: Indicates the client.
+        :param vo: The VO to authenticate into.
         """
 
         self.host = rucio_host
@@ -147,6 +157,7 @@ class BaseClient(object):
             raise MissingClientParameter('Section client and Option \'%s\' cannot be found in config file' % error.args[0])
 
         self.account = account
+        self.vo = vo
         self.ca_cert = ca_cert
         self.auth_type = auth_type
         self.creds = creds
@@ -162,7 +173,7 @@ class BaseClient(object):
         self.auth_oidc_refresh_before_exp = config_get_int('client', 'auth_oidc_refresh_before_exp', False, 20)
 
         if auth_type is None:
-            LOG.debug('no auth_type passed. Trying to get it from the environment variable RUCIO_AUTH_TYPE and config file.')
+            LOG.debug('No auth_type passed. Trying to get it from the environment variable RUCIO_AUTH_TYPE and config file.')
             if 'RUCIO_AUTH_TYPE' in environ:
                 if environ['RUCIO_AUTH_TYPE'] not in ['userpass', 'x509', 'x509_proxy', 'gss', 'ssh', 'saml', 'oidc']:
                     raise MissingClientParameter('Possible RUCIO_AUTH_TYPE values: userpass, x509, x509_proxy, gss, ssh, saml, oidc, vs. ' + environ['RUCIO_AUTH_TYPE'])
@@ -196,7 +207,7 @@ class BaseClient(object):
                 self.creds['oidc_polling'] = config_get_bool('client', 'oidc_polling', False, False)
 
         if not self.creds:
-            LOG.debug('no creds passed. Trying to get it from the config file.')
+            LOG.debug('No creds passed. Trying to get it from the config file.')
             self.creds = {}
             try:
                 if self.auth_type in ['userpass', 'saml']:
@@ -208,7 +219,7 @@ class BaseClient(object):
                 elif self.auth_type == 'x509_proxy':
                     try:
                         self.creds['client_proxy'] = path.abspath(path.expanduser(path.expandvars(config_get('client', 'client_x509_proxy'))))
-                    except NoOptionError as error:
+                    except NoOptionError:
                         # Recreate the classic GSI logic for locating the proxy:
                         # - $X509_USER_PROXY, if it is set.
                         # - /tmp/x509up_u`id -u` otherwise.
@@ -238,16 +249,17 @@ class BaseClient(object):
             raise ClientProtocolNotSupported('\'%s\' not supported' % auth_scheme)
 
         if (rucio_scheme == 'https' or auth_scheme == 'https') and ca_cert is None:
-            LOG.debug('no ca_cert passed. Trying to get it from the config file.')
+            LOG.debug('HTTPS is required, but no ca_cert was passed. Trying to get it from the config file.')
             try:
                 self.ca_cert = path.expandvars(config_get('client', 'ca_cert'))
-            except (NoOptionError, NoSectionError) as error:
-                raise MissingClientParameter('Option \'%s\' cannot be found in config file' % error.args[0])
+            except (NoOptionError, NoSectionError):
+                LOG.debug('No ca_cert found in configuration. Falling back to Mozilla default CA bundle (certifi).')
+                self.ca_cert = True
 
         self.list_hosts = [self.host]
 
         if account is None:
-            LOG.debug('no account passed. Trying to get it from the config file.')
+            LOG.debug('No account passed. Trying to get it from the config file.')
             try:
                 self.account = environ['RUCIO_ACCOUNT']
             except KeyError:
@@ -255,13 +267,28 @@ class BaseClient(object):
                     self.account = config_get('client', 'account')
                 except (NoOptionError, NoSectionError):
                     raise MissingClientParameter('Option \'account\' cannot be found in config file and RUCIO_ACCOUNT is not set.')
-        # if token file path is defined in the rucio.cfg file, use that file
+
+        if vo is None:
+            LOG.debug('No VO passed. Trying to get it from environment variable RUCIO_VO.')
+            try:
+                self.vo = environ['RUCIO_VO']
+            except KeyError:
+                LOG.debug('No VO found. Trying to get it from the config file.')
+                try:
+                    self.vo = config_get('client', 'vo')
+                except (NoOptionError, NoSectionError):
+                    LOG.debug('No VO found. Using default VO.')
+                    self.vo = 'def'
+
+        # if token file path is defined in the rucio.cfg file, use that file. Currently this prevents authenticating as another user or VO.
         if self.auth_token_file_path:
             self.token_file = self.auth_token_file_path
             self.token_path = '/'.join(self.token_file.split('/')[:-1])
             self.token_exp_epoch_file = self.token_path + '/' + self.TOKEN_EXP_PREFIX + self.account
         else:
             self.token_path = self.TOKEN_PATH_PREFIX + self.account
+            if self.vo != 'def':
+                self.token_path += '@%s' % self.vo
             self.token_file = self.token_path + '/' + self.TOKEN_PREFIX + self.account
             self.token_exp_epoch_file = self.token_path + '/' + self.TOKEN_EXP_PREFIX + self.account
 
@@ -328,7 +355,7 @@ class BaseClient(object):
         :param params: (optional) Dictionary or bytes to be sent in the url query string.
         :return: the HTTP return body.
         """
-        hds = {'X-Rucio-Auth-Token': self.auth_token, 'X-Rucio-Account': self.account,
+        hds = {'X-Rucio-Auth-Token': self.auth_token, 'X-Rucio-Account': self.account, 'X-Rucio-VO': self.vo,
                'Connection': 'Keep-Alive', 'User-Agent': self.user_agent,
                'X-Rucio-Script': self.script_id}
 
@@ -372,7 +399,11 @@ class BaseClient(object):
         :returns: True if the token was successfully received. False otherwise.
         """
 
-        headers = {'X-Rucio-Account': self.account, 'X-Rucio-Username': self.creds['username'], 'X-Rucio-Password': self.creds['password']}
+        headers = {'X-Rucio-VO': self.vo,
+                   'X-Rucio-Account': self.account,
+                   'X-Rucio-Username': self.creds['username'],
+                   'X-Rucio-Password': self.creds['password']}
+
         url = build_url(self.auth_host, path='auth/userpass')
 
         result = None
@@ -382,12 +413,11 @@ class BaseClient(object):
                 break
             except ConnectionError as error:
                 LOG.warning('ConnectionError: ' + str(error))
-                self.ca_cert = False
                 if retry > self.request_retries:
                     raise
 
         if not result or 'result' not in locals():
-            LOG.error('cannot get auth_token')
+            LOG.error('Cannot retrieve authentication token!')
             return False
 
         if result.status_code != codes.ok:  # pylint: disable-msg=E1101
@@ -397,7 +427,6 @@ class BaseClient(object):
             raise exc_cls(exc_msg)
 
         self.auth_token = result.headers['x-rucio-auth-token']
-        LOG.debug('got new token')
         return True
 
     def __refresh_token_OIDC(self):
@@ -428,7 +457,8 @@ class BaseClient(object):
         else:
             return False
 
-        headers = {'X-Rucio-Account': self.account,
+        headers = {'X-Rucio-VO': self.vo,
+                   'X-Rucio-Account': self.account,
                    'X-Rucio-Auth-Token': self.auth_token}
 
         for retry in range(self.AUTH_RETRIES + 1):
@@ -463,7 +493,6 @@ class BaseClient(object):
                 break
             except RequestException:
                 LOG.warning('RequestException: %s', str(traceback.format_exc()))
-                self.ca_cert = False
                 if retry > self.request_retries:
                     raise
 
@@ -481,7 +510,8 @@ class BaseClient(object):
         :returns: True if the token was successfully received. False otherwise.
         """
         oidc_scope = str(self.creds['oidc_scope'])
-        headers = {'X-Rucio-Account': self.account,
+        headers = {'X-Rucio-VO': self.vo,
+                   'X-Rucio-Account': self.account,
                    'X-Rucio-Client-Authorize-Auto': str(self.creds['oidc_auto']),
                    'X-Rucio-Client-Authorize-Polling': str(self.creds['oidc_polling']),
                    'X-Rucio-Client-Authorize-Scope': str(self.creds['oidc_scope']),
@@ -533,7 +563,7 @@ class BaseClient(object):
                             get_input = input
                             # if Python version <= 2.7 use raw_input
                             if sys.version_info[:2] <= (2, 7):
-                                get_input = raw_input
+                                get_input = raw_input  # noqa: F821 pylint: disable=undefined-variable
                             fetchcode = get_input()
                             fetch_url = build_url(self.auth_host, path='auth/oidc_redirect', params=fetchcode)
                             result = self.session.get(fetch_url, headers=headers, verify=self.ca_cert)
@@ -578,12 +608,11 @@ class BaseClient(object):
                 break
             except RequestException:
                 LOG.warning('RequestException: %s', str(traceback.format_exc()))
-                self.ca_cert = False
                 if retry > self.request_retries:
                     raise
 
         if not result or 'result' not in locals():
-            LOG.error('cannot get auth_token')
+            LOG.error('Cannot retrieve authentication token!')
             return False
 
         if result.status_code != codes.ok:  # pylint: disable-msg=E1101
@@ -612,7 +641,7 @@ class BaseClient(object):
         :returns: True if the token was successfully received. False otherwise.
         """
 
-        headers = {'X-Rucio-Account': self.account}
+        headers = {'X-Rucio-Account': self.account, 'X-Rucio-VO': self.vo}
 
         client_cert = None
         client_key = None
@@ -639,14 +668,12 @@ class BaseClient(object):
         result = None
         for retry in range(self.AUTH_RETRIES + 1):
             try:
-                result = self.session.get(url, headers=headers, cert=cert,
-                                          verify=self.ca_cert)
+                result = self.session.get(url, headers=headers, cert=cert, verify=self.ca_cert)
                 break
             except ConnectionError as error:
                 if 'alert certificate expired' in str(error):
                     raise CannotAuthenticate(str(error))
                 LOG.warning('ConnectionError: ' + str(error))
-                self.ca_cert = False
                 if retry > self.request_retries:
                     raise
 
@@ -663,7 +690,6 @@ class BaseClient(object):
             raise exc_cls(exc_msg)
 
         self.auth_token = result.headers['x-rucio-auth-token']
-        LOG.debug('got new token')
         return True
 
     def __get_token_ssh(self):
@@ -672,7 +698,7 @@ class BaseClient(object):
 
         :returns: True if the token was successfully received. False otherwise.
         """
-        headers = {'X-Rucio-Account': self.account}
+        headers = {'X-Rucio-Account': self.account, 'X-Rucio-VO': self.vo}
 
         private_key_path = self.creds['ssh_private_key']
         if not path.exists(private_key_path):
@@ -693,7 +719,6 @@ class BaseClient(object):
                 if 'alert certificate expired' in str(error):
                     raise CannotAuthenticate(str(error))
                 LOG.warning('ConnectionError: ' + str(error))
-                self.ca_cert = False
                 if retry > self.request_retries:
                     raise
 
@@ -727,12 +752,11 @@ class BaseClient(object):
                 if 'alert certificate expired' in str(error):
                     raise CannotAuthenticate(str(error))
                 LOG.warning('ConnectionError: ' + str(error))
-                self.ca_cert = False
                 if retry > self.request_retries:
                     raise
 
         if not result:
-            LOG.error('cannot get auth_token')
+            LOG.error('Cannot retrieve authentication token!')
             return False
 
         if result.status_code != codes.ok:   # pylint: disable-msg=E1101
@@ -742,7 +766,6 @@ class BaseClient(object):
             raise exc_cls(exc_msg)
 
         self.auth_token = result.headers['x-rucio-auth-token']
-        LOG.debug('got new token')
         return True
 
     def __get_token_gss(self):
@@ -754,7 +777,7 @@ class BaseClient(object):
         if not EXTRA_MODULES['requests_kerberos']:
             raise MissingModuleException('The requests-kerberos module is not installed.')
 
-        headers = {'X-Rucio-Account': self.account}
+        headers = {'X-Rucio-Account': self.account, 'X-Rucio-VO': self.vo}
         url = build_url(self.auth_host, path='auth/gss')
 
         result = None
@@ -765,12 +788,11 @@ class BaseClient(object):
                 break
             except ConnectionError as error:
                 LOG.warning('ConnectionError: ' + str(error))
-                self.ca_cert = False
                 if retry > self.request_retries:
                     raise
 
         if not result:
-            LOG.error('cannot get auth_token')
+            LOG.error('Cannot retrieve authentication token!')
             return False
 
         if result.status_code != codes.ok:   # pylint: disable-msg=E1101
@@ -780,7 +802,6 @@ class BaseClient(object):
             raise exc_cls(exc_msg)
 
         self.auth_token = result.headers['x-rucio-auth-token']
-        LOG.debug('got new token')
         return True
 
     def __get_token_saml(self):
@@ -790,7 +811,7 @@ class BaseClient(object):
         :returns: True if the token was successfully received. False otherwise.
         """
 
-        headers = {'X-Rucio-Account': self.account}
+        headers = {'X-Rucio-Account': self.account, 'X-Rucio-VO': self.vo}
         userpass = {'username': self.creds['username'], 'password': self.creds['password']}
         url = build_url(self.auth_host, path='auth/saml')
 
@@ -808,12 +829,11 @@ class BaseClient(object):
                 break
             except ConnectionError as error:
                 LOG.warning('ConnectionError: ' + str(error))
-                self.ca_cert = False
                 if retry > self.request_retries:
                     raise
 
         if not result or 'result' not in locals():
-            LOG.error('cannot get auth_token')
+            LOG.error('Cannot retrieve authentication token!')
             return False
 
         if result.status_code != codes.ok:  # pylint: disable-msg=E1101
@@ -823,7 +843,6 @@ class BaseClient(object):
             raise exc_cls(exc_msg)
 
         self.auth_token = result.headers['X-Rucio-Auth-Token']
-        LOG.debug('got new token')
         return True
 
     def __get_token(self):
