@@ -19,12 +19,13 @@
 # - Martin Barisits <martin.barisits@cern.ch>, 2013-2018
 # - Yun-Pin Sun <winter0128@gmail.com>, 2013
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2013
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2015
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2020
 # - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2014-2018
 # - Brian Bockelman <bbockelm@cse.unl.edu>, 2018
 # - Eric Vaandering <ericvaandering@gmail.com>, 2018
 # - Asket Agarwal <asket.agarwal96@gmail.com>, 2018
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 #
 # PY3K COMPATIBLE
 
@@ -51,9 +52,9 @@ class DIDClient(BaseClient):
     ARCHIVES_BASEURL = 'archives'
 
     def __init__(self, rucio_host=None, auth_host=None, account=None, ca_cert=None,
-                 auth_type=None, creds=None, timeout=600, user_agent='rucio-clients'):
+                 auth_type=None, creds=None, timeout=600, user_agent='rucio-clients', vo=None):
         super(DIDClient, self).__init__(rucio_host, auth_host, account, ca_cert,
-                                        auth_type, creds, timeout, user_agent)
+                                        auth_type, creds, timeout, user_agent, vo=vo)
 
     def list_dids(self, scope, filters, type='collection', long=False, recursive=False):
         """
@@ -88,6 +89,38 @@ class DIDClient(BaseClient):
             exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
+    def list_dids_extended(self, scope, filters, type='collection', long=False, recursive=False):
+        """
+        List all data identifiers in a scope which match a given pattern. Extended version that goes through plugin mechanism.
+
+        :param scope: The scope name.
+        :param filters: A dictionary of key/value pairs like {'name': 'file_name','rse-expression': 'tier0'}.
+        :param type: The type of the did: 'all'(container, dataset or file)|'collection'(dataset or container)|'dataset'|'container'|'file'
+        :param long: Long format option to display more information for each DID.
+        :param recursive: Recursively list DIDs content.
+        """
+        path = '/'.join([self.DIDS_BASEURL, quote_plus(scope), 'dids', 'search_extended'])
+        payload = {}
+        if long:
+            payload['long'] = 1
+
+        for k, v in list(filters.items()):
+            if k in ('created_before', 'created_after'):
+                payload[k] = date_to_str(v)
+            else:
+                payload[k] = v
+        payload['type'] = type
+        payload['recursive'] = recursive
+
+        url = build_url(choice(self.list_hosts), path=path, params=payload)
+        r = self._send_request(url, type='GET')
+        if r.status_code == codes.ok:
+            dids = self._load_json_data(r)
+            return dids
+        else:
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
+            raise exc_cls(exc_msg)
+
     def add_did(self, scope, name, type, statuses=None, meta=None, rules=None, lifetime=None, dids=None, rse=None):
         """
         Add data identifier for a dataset or container.
@@ -373,15 +406,18 @@ class DIDClient(BaseClient):
             exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
-    def get_did(self, scope, name):
+    def get_did(self, scope, name, dynamic=False):
         """
         Retrieve a single data identifier.
 
         :param scope: The scope name.
         :param name: The data identifier name.
+        :param dynamic: Calculate sizes dynamically when True
         """
 
         path = '/'.join([self.DIDS_BASEURL, quote_plus(scope), quote_plus(name)])
+        if dynamic:
+            path += '?dynamic=True'
         url = build_url(choice(self.list_hosts), path=path)
         r = self._send_request(url, type='GET')
         if r.status_code == codes.ok:
@@ -390,7 +426,7 @@ class DIDClient(BaseClient):
             exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
-    def get_metadata(self, scope, name):
+    def get_metadata(self, scope, name, plugin='DID_COLUMN'):
         """
         Get data identifier metadata
 
@@ -399,7 +435,9 @@ class DIDClient(BaseClient):
         """
         path = '/'.join([self.DIDS_BASEURL, quote_plus(scope), quote_plus(name), 'meta'])
         url = build_url(choice(self.list_hosts), path=path)
-        r = self._send_request(url, type='GET')
+        payload = {}
+        payload['plugin'] = plugin
+        r = self._send_request(url, type='GET', params=payload)
         if r.status_code == codes.ok:
             meta = self._load_json_data(r)
             return next(meta)
@@ -407,6 +445,20 @@ class DIDClient(BaseClient):
             exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
             raise exc_cls(exc_msg)
 
+    def get_metadata_bulk(self, dids):
+        """
+        Bulk get data identifier metadata
+        :param dids: A list of dids.
+        """
+        data = {'dids': dids}
+        path = '/'.join([self.DIDS_BASEURL, 'bulkmeta'])
+        url = build_url(choice(self.list_hosts), path=path)
+        r = self._send_request(url, type='POST', data=dumps(data))
+        if r.status_code == codes.ok:
+            return self._load_json_data(r)
+        exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
+        raise exc_cls(exc_msg)
+
     def set_metadata(self, scope, name, key, value, recursive=False):
         """
         Set data identifier metadata
@@ -462,10 +514,10 @@ class DIDClient(BaseClient):
         :param name: The data identifier.
         :param key: the key.
         """
-        path = '/'.join([self.DIDS_BASEURL, quote_plus(scope), quote_plus(name), 'meta', key])
-        url = build_url(choice(self.list_hosts), path=path)
-        r = self._send_request(url, type='DEL')
+        path = '/'.join([self.DIDS_BASEURL, quote_plus(scope), quote_plus(name), 'meta'])
+        url = build_url(choice(self.list_hosts), path=path, params={'key': key})
 
+        r = self._send_request(url, type='DEL')
         if r.status_code == codes.ok:
             return True
         else:
@@ -629,59 +681,6 @@ class DIDClient(BaseClient):
         exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
         raise exc_cls(exc_msg)
 
-    def add_did_meta(self, scope, name, meta):
-        """
-        Insert metadata to the json column of a did, updates key if already present
-
-        :param scope: the scope of did
-        :param name: the name of the did
-        :param meta: the metadata to be inserted or updated(in json format)
-        """
-        path = '/'.join([self.DIDS_BASEURL, quote_plus(scope), quote_plus(name), 'did_meta'])
-        url = build_url(choice(self.list_hosts), path=path)
-        data = dumps(meta)
-        r = self._send_request(url, type='POST', data=data)
-        if r.status_code == codes.created:
-            return True
-        else:
-            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
-            raise exc_cls(exc_msg)
-
-    def delete_did_meta(self, scope, name, key):
-        """
-        Delete a key from the metadata column
-
-        :param scope: the scope of did
-        :param name: the name of the did
-        :param key: the key to be deleted
-        """
-        path = '/'.join([self.DIDS_BASEURL, quote_plus(scope), quote_plus(name), 'did_meta'])
-        url = build_url(choice(self.list_hosts), path=path, params={'key': key})
-        r = self._send_request(url, type='DEL')
-
-        if r.status_code == codes.ok:
-            print(r.text)
-            return True
-        else:
-            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
-            raise exc_cls(exc_msg)
-
-    def get_did_meta(self, scope, name):
-        """
-        Get all metadata for a given did
-        :param scope: the scope of did
-        :param name: the name of the did
-        """
-        path = '/'.join([self.DIDS_BASEURL, quote_plus(scope), quote_plus(name), 'did_meta'])
-        url = build_url(choice(self.list_hosts), path=path)
-        r = self._send_request(url, type='GET')
-        if r.status_code == codes.ok:
-            meta = self._load_json_data(r)
-            return next(meta)
-        else:
-            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code, data=r.content)
-            raise exc_cls(exc_msg)
-
     def list_dids_by_meta(self, scope=None, select={}):
         """
         Gets all dids matching the values of the provided metadata keys
