@@ -1,4 +1,5 @@
-# Copyright 2012-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2012-2020 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,25 +16,33 @@
 # Authors:
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2012
 # - Angelos Molfetas <Angelos.Molfetas@cern.ch>, 2012
-# - Vincent Garonne <vgaronne@gmail.com>, 2012-2018
-# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2014-2018
-# - Fernando Lopez <fernando.e.lopez@gmail.com>, 2015
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
+# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2014-2018
+# - Fernando López <felopez@cern.ch>, 2015
 # - Martin Barisits <martin.barisits@cern.ch>, 2017
-
-from paste.fixture import TestApp
-from random import choice
-from six import PY3
-from string import ascii_uppercase
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 
 import contextlib
 import os
 import subprocess
 import tempfile
+from random import choice
+from string import ascii_uppercase
+
+import pytest
+from paste.fixture import TestApp
+from six import PY3
 
 from rucio.client.accountclient import AccountClient
 from rucio.common import exception
+from rucio.common.config import config_get, config_get_bool
 from rucio.common.utils import generate_uuid as uuid
 
+skip_rse_tests_with_accounts = pytest.mark.skipif(not os.path.exists('etc/rse-accounts.cfg'), reason='fails if no rse-accounts.cfg found')
+
 
 def execute(cmd):
     """
@@ -83,9 +92,15 @@ def get_auth_token(account, username, password):
     :param password: the password linked to the account
     :returns: the authentication token
     """
+    if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+        vo_header = {'X-Rucio-VO': config_get('client', 'vo', raise_exception=False, default='tst')}
+    else:
+        vo_header = {}
+
     from rucio.web.rest.authentication import APP as auth_app
     mw = []
     header = {'Rucio-Account': account, 'Rucio-Username': username, 'Rucio-Password': password}
+    header.update(vo_header)
     r1 = TestApp(auth_app.wsgifunc(*mw)).get('/userpass', headers=header, expect_errors=True)
     token = str(r1.header('Rucio-Auth-Token'))
     return token
@@ -96,7 +111,7 @@ def account_name_generator():
 
     :returns: A random account name
     """
-    return 'jdoe-' + str(uuid()).lower()[:20]
+    return 'jdoe-' + str(uuid()).lower()[:16]
 
 
 def scope_name_generator():
@@ -104,7 +119,7 @@ def scope_name_generator():
 
     :returns: A random scope name
     """
-    return 'mock_' + str(uuid()).lower()[:20]
+    return 'mock_' + str(uuid()).lower()[:16]
 
 
 def rse_name_generator(size=10):
